'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var fsp = require('fs/promises');
var consola = require('consola');
var fs = require('fs');
var sass = require('sass');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n["default"] = e;
    return Object.freeze(n);
}

var fsp__default = /*#__PURE__*/_interopDefaultLegacy(fsp);
var consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);
var sass__namespace = /*#__PURE__*/_interopNamespace(sass);

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function isObject(val) {
  return val !== null && typeof val === "object";
}
function _defu(baseObj, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObj, {}, namespace, merger);
  }
  const obj = Object.assign({}, defaults);
  for (const key in baseObj) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const val = baseObj[key];
    if (val === null || val === void 0) {
      continue;
    }
    if (merger && merger(obj, key, val, namespace)) {
      continue;
    }
    if (Array.isArray(val) && Array.isArray(obj[key])) {
      obj[key] = val.concat(obj[key]);
    } else if (isObject(val) && isObject(obj[key])) {
      obj[key] = _defu(val, obj[key], (namespace ? `${namespace}.` : "") + key.toString(), merger);
    } else {
      obj[key] = val;
    }
  }
  return obj;
}
function createDefu(merger) {
  return (...args) => args.reduce((p, c) => _defu(p, c, "", merger), {});
}
const defu = createDefu();

function getKey(str) {
    return str;
}
function getJsValue(str) {
    return `withOpacityValue('${str}')`;
}
function getScssValue(str) {
    return `rgb(var(${str}))`;
}
function getOption(option) {
    var _a;
    const filesDefault = {
        extendColors: {
            getVarName: getKey,
            getVarValue: getJsValue
        },
        variables: {
            getVarName: getKey,
            getVarValue: getScssValue
        },
        root: {
            replacement: {
            // filepath: '{{filepath}}',
            // variableName: '{{variableName}}'
            }
        },
        export: {
            replacement: {
            // filepath: '{{filepath}}',
            // variableName: '{{variableName}}'
            }
        },
        util: true
    };
    return {
        entryPoint: option.entryPoint,
        outdir: (_a = option.outdir) !== null && _a !== void 0 ? _a : '.',
        files: defu(option.files, filesDefault),
        sassOptions: {},
        outputFileSystem: fsp__default["default"],
        write: true
    };
}

function cmkdir(dir) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield fsp__default["default"].access(dir, fs.constants.F_OK);
        }
        catch (error) {
            yield fsp__default["default"].mkdir(dir, {
                recursive: true
            });
        }
    });
}
function renderTemplete(src, replacement) {
    return __awaiter(this, void 0, void 0, function* () {
        let t = yield fsp__default["default"].readFile(src, 'utf-8');
        if (replacement) {
            const replacements = Object.entries(replacement);
            for (let i = 0; i < replacements.length; i++) {
                const [key, value] = replacements[i];
                t = t.replace(key, value);
            }
        }
        return t;
    });
}

// The asynchronous variants are much slower
function exposeScssVariable(exposeFilePath, options) {
    const exposeAarry = [];
    const defaultOption = {
        functions: {
            'expose($map)': (args) => {
                const value = args[0].assertMap('map');
                const map = value.contents;
                exposeAarry.push(map);
                return value;
            }
        }
    };
    sass__namespace.compile(exposeFilePath, defu(options, defaultOption));
    return exposeAarry;
}

function generate(option) {
    return __awaiter(this, void 0, void 0, function* () {
        const opt = getOption(option);
        const fsp = opt.outputFileSystem;
        const { entryPoint, outdir, files, write } = opt;
        const exposeAarry = exposeScssVariable(entryPoint);
        const targetDir = path.dirname(entryPoint);
        const absOutdir = path.resolve(targetDir, outdir);
        const mergedMap = {};
        const microtaskProducer = [];
        const result = {
            js: {},
            scss: {}
        };
        for (let i = 0; i < exposeAarry.length; i++) {
            const map = exposeAarry[i];
            for (const [key, color] of map) {
                mergedMap[key.text] = `${color.red} ${color.green} ${color.blue}`; // ${color.alpha}
            }
        }
        const keys = Object.keys(mergedMap);
        yield cmkdir(absOutdir);
        consola__default["default"].info('[Output Dir]: ' + absOutdir);
        if (files.variables !== false) {
            const { getVarName, getVarValue, outfile } = files.variables;
            // scss
            // ${{ removeColorPrefix(k) }}:{{ scssFilterShadow(k) }};
            const scssResult = keys
                .map((x) => {
                return `$${getVarName(x)}:${getVarValue(x)};`;
            })
                .join('\n');
            result.scss.variables = scssResult;
            if (write) {
                microtaskProducer.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield fsp.writeFile(outfile !== null && outfile !== void 0 ? outfile : path.resolve(absOutdir, 'variables.scss'), scssResult);
                    consola__default["default"].success('[variables.scss] generate Successfully!');
                }));
            }
        }
        if (files.extendColors !== false) {
            // js:
            // '{{ removeColorPrefix(k) }}':{{ jsFilterShadow(k) }},
            const filename = 'extendColors.js';
            const { getVarName, getVarValue, outfile } = files.extendColors;
            const jsResult = keys
                .map((x) => {
                return `'${getVarName(x)}': ${getVarValue(x)},`;
            })
                .join('\n    ');
            result.js.extendColors = jsResult;
            const extendColorsTemplete = yield renderTemplete(path.resolve(__dirname, `./t/js/${filename}`), {
                '/* {{placeholder}} */': jsResult
            });
            if (write) {
                microtaskProducer.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield fsp.writeFile(outfile !== null && outfile !== void 0 ? outfile : path.resolve(absOutdir, filename), extendColorsTemplete);
                    consola__default["default"].success(`[${filename}] generate Successfully!`);
                }));
            }
        }
        function handleScssFile(key, filename) {
            return __awaiter(this, void 0, void 0, function* () {
                const file = files[key];
                if (file !== false) {
                    const src = path.resolve(__dirname, `./t/scss/${filename}`);
                    const dest = path.resolve(absOutdir, filename);
                    if (file === true) {
                        yield fsp.copyFile(src, dest);
                    }
                    else {
                        const { replacement, outfile } = file;
                        if (replacement) {
                            const content = yield renderTemplete(src, replacement);
                            yield fsp.writeFile(outfile !== null && outfile !== void 0 ? outfile : dest, content, 'utf-8');
                        }
                        else {
                            yield fsp.copyFile(src, outfile !== null && outfile !== void 0 ? outfile : dest);
                        }
                    }
                    consola__default["default"].success(`[${filename}] generate Successfully!`);
                }
            });
        }
        microtaskProducer.push(() => handleScssFile('util', 'util.scss'));
        microtaskProducer.push(() => handleScssFile('export', 'export.scss'));
        microtaskProducer.push(() => handleScssFile('root', 'root.scss'));
        yield Promise.all(microtaskProducer.map((fn) => fn()));
        return result;
    });
}

exports.exposeScssVariable = exposeScssVariable;
exports.generate = generate;
